@@ -4,10 +4,10 @@
 #include <stdexcept>
 #include <string>
 
-#include "configparser.h"
-
 namespace newsboat {
 
+enum class ActionHandlerStatus;
+
 class ConfigHandlerException : public std::exception {
 public:
 	explicit ConfigHandlerException(const std::string& emsg)
