@@ -1,14 +1,15 @@
 #ifndef NEWSBOAT_DIALOGSFORMACTION_H_
 #define NEWSBOAT_DIALOGSFORMACTION_H_
 
-#include "formaction.h"
+#include "listformaction.h"
 #include "listwidget.h"
+#include "regexmanager.h"
 
 namespace newsboat {
 
-class DialogsFormAction : public FormAction {
+class DialogsFormAction : public ListFormAction {
 public:
-	DialogsFormAction(View*, std::string formstr, ConfigContainer* cfg);
+	DialogsFormAction(View*, std::string formstr, ConfigContainer* cfg, RegexManager& r);
 	~DialogsFormAction() override;
 	void prepare() override;
 	void init() override;
@@ -28,11 +29,9 @@ protected:
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	void update_heading();
-
-	ListWidget dialogs_list;
 };
 
 } // namespace newsboat
