@@ -34,14 +34,14 @@ protected:
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	void update_title(const std::string& working_directory);
 
-	void add_directory(ListFormatter& listfmt,
-		std::vector<file_system::FileSystemEntry>& id_at_position,
+	void add_directory(std::vector<file_system::FileSystemEntry>& id_at_position,
 		std::string dirname);
 	std::vector<file_system::FileSystemEntry> id_at_position;
+	std::vector<std::string> lines;
 
 	std::string get_formatted_dirname(std::string dirname, mode_t mode);
 
