@@ -8,11 +8,12 @@ namespace newsboat {
 
 class HelpFormAction : public FormAction {
 public:
-	HelpFormAction(View*, std::string formstr, ConfigContainer* cfg);
+	HelpFormAction(View*, std::string formstr, ConfigContainer* cfg,
+		const std::string& ctx);
 	~HelpFormAction() override;
 	void prepare() override;
 	void init() override;
-	KeyMapHintEntry* get_keymap_hint() override;
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
 	std::string id() const override
 	{
 		return "help";
@@ -20,17 +21,22 @@ public:
 	std::string title() override;
 
 	void finished_qna(Operation op) override;
-	void set_context(const std::string& ctx);
+
+protected:
+	std::string main_widget() const override
+	{
+		return "helptext";
+	}
 
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	std::string make_colorstring(const std::vector<std::string>& colors);
 	bool quit;
 	bool apply_search;
 	std::string searchphrase;
-	std::string context;
+	const std::string context;
 	TextviewWidget textview;
 };
 
