@@ -7,18 +7,20 @@
 
 #include "formaction.h"
 #include "listwidget.h"
+#include "regexmanager.h"
 
 namespace newsboat {
 
 class ListFormAction : public FormAction {
 public:
-	ListFormAction(View*, std::string formstr, std::string list_name,
-		ConfigContainer* cfg);
+	ListFormAction(View* v, const std::string& context, std::string formstr,
+		std::string list_name,
+		ConfigContainer* cfg, RegexManager& r);
 
 protected:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
 	nonstd::optional<std::uint8_t> open_unread_items_in_browser(
 		std::shared_ptr<RssFeed> feed,
 		bool markread);
