@@ -16,8 +16,8 @@ public:
 	~UrlViewFormAction() override;
 	void prepare() override;
 	void init() override;
-	KeyMapHintEntry* get_keymap_hint() override;
-	void set_links(const std::vector<LinkPair>& l)
+	const std::vector<KeyMapHintEntry>& get_keymap_hint() const override;
+	void set_links(const Links& l)
 	{
 		links = l;
 	}
@@ -28,11 +28,20 @@ public:
 	std::string title() override;
 	void handle_cmdline(const std::string& cmd) override;
 
+protected:
+	std::string main_widget() const override
+	{
+		return "urls";
+	}
+
 private:
 	bool process_operation(Operation op,
-		bool automatic = false,
-		std::vector<std::string>* args = nullptr) override;
-	std::vector<LinkPair> links;
+		const std::vector<std::string>& args,
+		BindingType bindingType = BindingType::BindKey) override;
+	void open_current_position_in_browser(bool interactive);
+	void update_heading();
+
+	Links links;
 	bool quit;
 	std::shared_ptr<RssFeed> feed;
 	ListWidget urls_list;
