@@ -76,14 +76,14 @@ public:
 		const std::string& searchphrase = "");
 	void push_empty_formaction();
 	void push_help();
-	void push_urlview(const std::vector<LinkPair>& links,
+	void push_urlview(const Links& links,
 		std::shared_ptr<RssFeed>& feed);
 	void push_searchresult(std::shared_ptr<RssFeed> feed,
 		const std::string& phrase = "");
 	void view_dialogs();
 
-	std::string run_filebrowser(const std::string& default_filename = "");
-	std::string run_dirbrowser();
+	nonstd::optional<std::string> run_filebrowser(const std::string& default_filename = "");
+	nonstd::optional<std::string> run_dirbrowser();
 	std::string select_tag(const std::string& current_tag);
 	std::string select_filter(
 		const std::vector<FilterNameExprPair>& filters);
