@@ -16,7 +16,7 @@ bool is_media_node(xmlNode* node)
 	return has_namespace(node, MEDIA_RSS_URI);
 }
 
-void parse_media_node(xmlNode* node, Item& it)
+void parse_media_node(xmlNode* node, Item& it, Enclosure* enclosure)
 {
 	if (node_is(node, "group", MEDIA_RSS_URI)) {
 		for (xmlNode* mnode = node->children; mnode != nullptr; mnode = mnode->next) {
@@ -27,24 +27,38 @@ void parse_media_node(xmlNode* node, Item& it)
 		Enclosure {
 			get_prop(node, "url"),
 			get_prop(node, "type"),
+			"",
+			"",
 		}
 		);
 		for (xmlNode* mnode = node->children; mnode != nullptr; mnode = mnode->next) {
-			parse_media_node(mnode, it);
+			parse_media_node(mnode, it, &it.enclosures.back());
 		}
 	} else if (node_is(node, "description", MEDIA_RSS_URI)) {
+		const std::string description = get_content(node);
 		const std::string type = get_prop(node, "type");
+		const std::string mime_type = (type == "html" ? "text/html" : "text/plain");
 		if (it.description.empty()) {
-			it.description = get_content(node);
-			if (type == "html") {
-				it.description_mime_type = "text/html";
-			} else {
-				it.description_mime_type = "text/plain";
-			}
+			it.description = description;
+			it.description_mime_type = mime_type;
+		}
+		if (enclosure) {
+			enclosure->description = description;
+			enclosure->description_mime_type = mime_type;
 		}
 	} else if (node_is(node, "title", MEDIA_RSS_URI)) {
+		const std::string title = get_content(node);
+		const std::string type = get_prop(node, "type");
+		const std::string mime_type = (type == "html" ? "text/html" : "text/plain");
 		if (it.title.empty()) {
-			it.title = get_content(node);
+			it.title = title;
+			it.title_type = type;
+		}
+		if (enclosure) {
+			if (enclosure->description.empty()) {
+				enclosure->description = title;
+				enclosure->description_mime_type = mime_type;
+			}
 		}
 	} else if (node_is(node, "player", MEDIA_RSS_URI)) {
 		if (it.link.empty()) {
