@@ -1,4 +1,4 @@
-use libc::c_char;
+use libc::{c_char, c_ulong};
 use libnewsboat::utils::{self, *};
 use std::ffi::{CStr, CString};
 use std::path::{Path, PathBuf};
@@ -84,7 +84,7 @@ mod bridged {
 
         fn podcast_mime_to_link_type(mime_type: &str, result: &mut i64) -> bool;
 
-        fn run_program(argv: &[&str], input: &str) -> String;
+        fn run_program(argv: &[&str], input: String) -> String;
 
         fn translit(tocode: &str, fromcode: &str) -> String;
         fn utf8_to_locale(text: &str) -> Vec<u8>;
@@ -102,7 +102,16 @@ mod bridged {
 }
 
 fn get_auth_method(method: &str) -> u64 {
-    utils::get_auth_method(method) as u64
+    let result: c_ulong = utils::get_auth_method(method);
+    #[cfg(target_pointer_width = "32")]
+    {
+        // On 32-bit platforms, upcast
+        result as u64
+    }
+    #[cfg(target_pointer_width = "64")]
+    {
+        result
+    }
 }
 
 fn run_interactively(command: &str, caller: &str, exit_code: &mut u8) -> bool {
