@@ -1,4 +1,4 @@
-use crate::htmlrenderer;
+use crate::links;
 use crate::logger::{self, Level};
 use libc::{
     c_char, c_int, c_ulong, c_void, close, execvp, exit, fork, size_t, waitpid, E2BIG, EILSEQ,
@@ -26,7 +26,7 @@ pub fn consolidate_whitespace(input: String) -> String {
 
     if let Some(found) = found {
         let (leading, rest) = input.split_at(found);
-        let lastchar = input.chars().rev().next().unwrap();
+        let lastchar = input.chars().next_back().unwrap();
 
         result.push_str(leading);
 
@@ -155,7 +155,7 @@ pub fn get_default_browser() -> String {
 
 pub fn md5hash(input: &str) -> String {
     let digest = md5::compute(input);
-    let hash = format!("{:x}", digest);
+    let hash = format!("{digest:x}");
     hash
 }
 
@@ -315,7 +315,7 @@ pub fn substr_with_width_stfl(string: &str, max_width: usize) -> String {
 }
 
 /// Remove all soft-hyphens as they can behave unpredictably (see
-/// https://github.com/akrennmair/newsbeuter/issues/259#issuecomment-259609490) and inadvertently
+/// <https://github.com/akrennmair/newsbeuter/issues/259#issuecomment-259609490>) and inadvertently
 /// render as hyphens
 pub fn remove_soft_hyphens(text: &mut String) {
     text.retain(|c| c != '\u{00AD}')
@@ -330,7 +330,7 @@ pub fn remove_soft_hyphens(text: &mut String) {
 ///
 /// 2. figuring out the `LinkType` for a particular enclosure, given its MIME type
 ///    (`utils::podcast_mime_to_link_type`).
-type MimeMatcher = (fn(&str) -> bool, htmlrenderer::LinkType);
+type MimeMatcher = (fn(&str) -> bool, links::LinkType);
 const PODCAST_MIME_TO_LINKTYPE: [MimeMatcher; 2] = [
     (
         |mime| {
@@ -339,12 +339,9 @@ const PODCAST_MIME_TO_LINKTYPE: [MimeMatcher; 2] = [
             // https://tools.ietf.org/html/rfc5334#section-10.1
             mime.starts_with("audio/") || mime == "application/ogg"
         },
-        htmlrenderer::LinkType::Audio,
-    ),
-    (
-        |mime| mime.starts_with("video/"),
-        htmlrenderer::LinkType::Video,
+        links::LinkType::Audio,
     ),
+    (|mime| mime.starts_with("video/"), links::LinkType::Video),
 ];
 
 /// Returns `true` if given MIME type is considered to be a podcast by Newsboat.
@@ -357,7 +354,7 @@ pub fn is_valid_podcast_type(mimetype: &str) -> bool {
 /// Converts podcast's MIME type into an HtmlRenderer's "link type"
 ///
 /// Returns None if given MIME type is not a podcast type. See `is_valid_podcast_type()`.
-pub fn podcast_mime_to_link_type(mime_type: &str) -> Option<htmlrenderer::LinkType> {
+pub fn podcast_mime_to_link_type(mime_type: &str) -> Option<links::LinkType> {
     PODCAST_MIME_TO_LINKTYPE
         .iter()
         .find_map(|(matcher, link_type)| {
@@ -468,7 +465,7 @@ pub fn run_command(cmd: &str, param: &str) {
     }
 }
 
-pub fn run_program(cmd_with_args: &[&str], input: &str) -> String {
+pub fn run_program(cmd_with_args: &[&str], input: String) -> String {
     if cmd_with_args.is_empty() {
         return String::new();
     }
@@ -485,19 +482,21 @@ pub fn run_program(cmd_with_args: &[&str], input: &str) -> String {
                 "utils::run_program: spawning a child for \"{:?}\" \
                  with input \"{}\" failed: {}",
                 cmd_with_args,
-                input,
+                &input,
                 error
             );
         })
         .and_then(|mut child| {
-            if let Some(stdin) = child.stdin.as_mut() {
-                if let Err(error) = stdin.write_all(input.as_bytes()) {
-                    log!(
-                        Level::Debug,
-                        "utils::run_program: failed to write to child's stdin: {}",
-                        error
-                    );
-                }
+            if let Some(mut stdin) = child.stdin.take() {
+                std::thread::spawn(move || {
+                    if let Err(error) = stdin.write_all(input.as_bytes()) {
+                        log!(
+                            Level::Debug,
+                            "utils::run_program: failed to write to child's stdin: {}",
+                            error
+                        );
+                    }
+                });
             }
 
             child
@@ -540,7 +539,7 @@ pub fn make_title(rs_str: String) -> String {
         .trim_end_matches(".htm");
 
     // 'title with dashes'
-    let result = result.replace('-', " ").replace('_', " ");
+    let result = result.replace(['-', '_'], " ");
 
     //'Title with dashes'
     //let result = "";
@@ -552,16 +551,23 @@ pub fn make_title(rs_str: String) -> String {
 
     // Un-escape any percent-encoding, e.g. "It%27s%202017%21" -> "It's
     // 2017!"
-    match unescape_url(result) {
+    let result = match unescape_url(result) {
         None => String::new(),
         Some(f) => f,
+    };
+
+    // If it contains only digits, assume it's an id, not a proper title
+    if result.chars().all(|c| c.is_ascii_digit()) {
+        String::new()
+    } else {
+        result
     }
 }
 
 /// Run the given command interactively with inherited stdin and stdout/stderr. Return the lowest
 /// 8 bits of its exit code, or `None` if the command failed to start.
 pub fn run_interactively(command: &str, caller: &str) -> Option<u8> {
-    log!(Level::Debug, &format!("{}: running `{}'", caller, command));
+    log!(Level::Debug, &format!("{caller}: running `{command}'"));
     Command::new("sh")
         .arg("-c")
         .arg(command)
@@ -569,7 +575,7 @@ pub fn run_interactively(command: &str, caller: &str) -> Option<u8> {
         .map_err(|err| {
             log!(
                 Level::Warn,
-                &format!("{}: Couldn't create child process: {}", caller, err)
+                &format!("{caller}: Couldn't create child process: {err}")
             );
         })
         .ok()
@@ -580,7 +586,7 @@ pub fn run_interactively(command: &str, caller: &str) -> Option<u8> {
 /// Run the given command non-interactively with closed stdin and stdout/stderr. Return the lowest
 /// 8 bits of its exit code, or `None` if the command failed to start.
 pub fn run_non_interactively(command: &str, caller: &str) -> Option<u8> {
-    log!(Level::Debug, &format!("{}: running `{}'", caller, command));
+    log!(Level::Debug, &format!("{caller}: running `{command}'"));
     Command::new("sh")
         .arg("-c")
         .arg(command)
@@ -591,7 +597,7 @@ pub fn run_non_interactively(command: &str, caller: &str) -> Option<u8> {
         .map_err(|err| {
             log!(
                 Level::Warn,
-                &format!("{}: Couldn't create child process: {}", caller, err)
+                &format!("{caller}: Couldn't create child process: {err}")
             );
         })
         .ok()
@@ -787,9 +793,11 @@ pub fn tokenize_quoted(line: &str, delimiters: &str) -> Vec<String> {
 
 /// The result of executing `extract_filter()`.
 pub struct FilterUrlParts {
+    #[allow(rustdoc::bare_urls)]
     /// "~/bin/foo.sh" in "filter:~/bin/foo.sh:https://example.com/news.atom"
     pub script_name: String,
 
+    #[allow(rustdoc::bare_urls)]
     /// "https://example.com/news.atom" in "filter:~/bin/foo.sh:https://example.com/news.atom"
     pub url: String,
 }
@@ -861,7 +869,7 @@ pub fn translit(tocode: &str, fromcode: &str) -> String {
         return tocode.to_string();
     }
 
-    let tocode_translit = format!("{}//TRANSLIT", tocode);
+    let tocode_translit = format!("{tocode}//TRANSLIT");
 
     unsafe {
         if STATE == TranslitState::Unknown {
@@ -887,15 +895,12 @@ pub fn translit(tocode: &str, fromcode: &str) -> String {
                         iconv_close(cd);
                     } else {
                         let errno = std::io::Error::last_os_error();
-                        eprintln!("iconv_open('{}', '{}') failed: {}", tocode, fromcode, errno);
+                        eprintln!("iconv_open('{tocode}', '{fromcode}') failed: {errno}");
                         std::process::abort();
                     }
                 } else {
                     let errno = std::io::Error::last_os_error();
-                    eprintln!(
-                        "iconv_open('{}', '{}') failed: {}",
-                        tocode_translit, fromcode, errno
-                    );
+                    eprintln!("iconv_open('{tocode_translit}', '{fromcode}') failed: {errno}");
                     std::process::abort();
                 }
             } else {
@@ -914,10 +919,6 @@ pub fn translit(tocode: &str, fromcode: &str) -> String {
 
 /// Converts `text` from encoding `fromcode` to encoding `tocode`.
 pub fn convert_text(text: &[u8], tocode: &str, fromcode: &str) -> Vec<u8> {
-    if tocode.to_lowercase() == fromcode.to_lowercase() {
-        return text.to_owned();
-    }
-
     let mut result = vec![];
 
     let tocode_translit = translit(tocode, fromcode);
@@ -1215,23 +1216,23 @@ mod tests {
     fn t_tokenize_quoted_interprets_double_backslash_as_literal_backslash() {
         assert_eq!(tokenize_quoted(r#""""#, ""), vec![""]);
 
-        assert_eq!(tokenize_quoted(r#""\\""#, ""), vec![r#"\"#]);
+        assert_eq!(tokenize_quoted(r#""\\""#, ""), vec![r"\"]);
 
         assert_eq!(tokenize_quoted(r##""#\\""##, ""), vec!["#\\"]);
 
         assert_eq!(tokenize_quoted(r#""'#\\'""#, ""), vec!["'#\\'"]);
 
-        assert_eq!(tokenize_quoted(r#""'#\\ \\'""#, ""), vec![r#"'#\ \'"#]);
+        assert_eq!(tokenize_quoted(r#""'#\\ \\'""#, ""), vec![r"'#\ \'"]);
 
-        assert_eq!(tokenize_quoted("\"\\\\\\\\", ""), vec![r#"\\"#]);
+        assert_eq!(tokenize_quoted("\"\\\\\\\\", ""), vec![r"\\"]);
 
-        assert_eq!(tokenize_quoted("\"\\\\\\\\\\\\", ""), vec![r#"\\\"#]);
+        assert_eq!(tokenize_quoted("\"\\\\\\\\\\\\", ""), vec![r"\\\"]);
 
-        assert_eq!(tokenize_quoted("\"\\\\\\\\\"", ""), vec![r#"\\"#]);
+        assert_eq!(tokenize_quoted("\"\\\\\\\\\"", ""), vec![r"\\"]);
 
-        assert_eq!(tokenize_quoted("\"\\\\\\\\\\\\\"", ""), vec![r#"\\\"#]);
+        assert_eq!(tokenize_quoted("\"\\\\\\\\\\\\\"", ""), vec![r"\\\"]);
 
-        assert_eq!(tokenize_quoted(r#""\\bgit\\b""#, ""), vec![r#"\bgit\b"#]);
+        assert_eq!(tokenize_quoted(r#""\\bgit\\b""#, ""), vec![r"\bgit\b"]);
 
         assert_eq!(
             tokenize_quoted(
@@ -1240,7 +1241,7 @@ mod tests {
             ),
             vec![
                 "browser",
-                r#"/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --app %u"#
+                r"/Applications/Google\ Chrome.app/Contents/MacOS/Google\ Chrome --app %u"
             ]
         );
     }
@@ -1286,7 +1287,7 @@ mod tests {
     #[test]
     fn t_tokenize_quoted_ignores_escaped_pound_sign_start_of_token() {
         assert_eq!(
-            tokenize_quoted(r#"one \# two three # ???"#, " "),
+            tokenize_quoted(r"one \# two three # ???", " "),
             vec!["one", "\\#", "two", "three"]
         );
     }
@@ -1578,7 +1579,7 @@ mod tests {
 
     #[test]
     fn t_podcast_mime_to_link_type() {
-        use crate::htmlrenderer::LinkType::*;
+        use crate::links::LinkType::*;
 
         assert_eq!(podcast_mime_to_link_type("audio/mpeg"), Some(Audio));
         assert_eq!(podcast_mime_to_link_type("audio/mp3"), Some(Audio));
@@ -1706,10 +1707,10 @@ mod tests {
     #[test]
     fn t_run_program() {
         let input1 = "this is a multine-line\ntest string";
-        assert_eq!(run_program(&["cat"], input1), input1);
+        assert_eq!(run_program(&["cat"], input1.to_owned()), input1);
 
         assert_eq!(
-            run_program(&["echo", "-n", "hello world"], ""),
+            run_program(&["echo", "-n", "hello world"], String::new()),
             "hello world"
         );
     }
@@ -1978,12 +1979,12 @@ mod tests {
         assert_eq!(strip_comments(&input), expected);
 
         // Escaped backtick inside backticks is not treated as closing
-        let expected = r#"some `other \` tricky # test` hehe"#;
+        let expected = r"some `other \` tricky # test` hehe";
         let input = expected.to_owned() + "#here goescomment";
         assert_eq!(strip_comments(&input), expected);
 
         // Ignores escaped # characters (\\#)
-        let expected = r#"one two \# three four"#;
+        let expected = r"one two \# three four";
         let input = expected.to_owned() + "# and a comment";
         assert_eq!(strip_comments(&input), expected);
     }
@@ -2036,7 +2037,7 @@ mod tests {
 
         let check = |fromcode, tocode| {
             let expected1 = tocode;
-            let expected2 = format!("{}//TRANSLIT", tocode);
+            let expected2 = format!("{tocode}//TRANSLIT");
 
             let actual = translit(tocode, fromcode);
 
@@ -2080,61 +2081,11 @@ mod tests {
     }
 
     #[test]
-    fn t_convert_text_returns_input_string_if_fromcode_and_tocode_are_literally_the_same() {
-        let inputs: &[&[u8]] = &[
-            &[0x81, 0x13, 0xa0],       // \x81 is not valid UTF-8
-            &[0x01],                   // incomplete UTF-16
-            &[0x01, 0x1f, 0x80, 0x9b], // those bytes are not defined in ISO-8859-1
-            &[0x7f, 0x1e, 0x03],       // these bytes are not defined in KOI8-R
-        ];
-
-        let codes = &["utf-8", "utf-16", "iso-8859-1", "koi8-r"];
-
-        for code in codes {
-            for input in inputs {
-                assert_eq!(convert_text(input, code, code), *input);
-            }
-        }
-    }
-
-    #[test]
-    fn t_convert_text_returns_input_string_if_fromcode_is_an_uppercase_of_tocode() {
-        let inputs: &[&[u8]] = &[
-            &[0x81, 0x13, 0xa0],       // \x81 is not valid UTF-8
-            &[0x01],                   // incomplete UTF-16
-            &[0x01, 0x1f, 0x80, 0x9b], // those bytes are not defined in ISO-8859-1
-            &[0x7f, 0x1e, 0x03],       // these bytes are not defined in KOI8-R
-        ];
-
-        let codes = &["utf-8", "utf-16", "iso-8859-1", "koi8-r"];
-
-        for code in codes {
-            for input in inputs {
-                let fromcode = code.to_uppercase();
-                let tocode = code;
-                assert_eq!(convert_text(input, tocode, &fromcode), *input);
-            }
-        }
-    }
-
-    #[test]
-    fn t_convert_text_returns_input_string_if_tocode_is_an_uppercase_of_fromcode() {
-        let inputs: &[&[u8]] = &[
-            &[0x81, 0x13, 0xa0],       // \x81 is not valid UTF-8
-            &[0x01],                   // incomplete UTF-16
-            &[0x01, 0x1f, 0x80, 0x9b], // those bytes are not defined in ISO-8859-1
-            &[0x7f, 0x1e, 0x03],       // these bytes are not defined in KOI8-R
-        ];
-
-        let codes = &["utf-8", "utf-16", "iso-8859-1", "koi8-r"];
-
-        for code in codes {
-            for input in inputs {
-                let fromcode = code;
-                let tocode = code.to_uppercase();
-                assert_eq!(convert_text(input, &tocode, fromcode), *input);
-            }
-        }
+    fn t_convert_text_returns_input_string_if_fromcode_and_tocode_are_the_same() {
+        // \x81 is not valid UTF-8
+        let input = &[0x81, 0x13, 0x41];
+        let expected = &[0x3f, 0x13, 0x41];
+        assert_eq!(convert_text(input, "UTF-8", "UTF-8"), expected);
     }
 
     #[test]
