@@ -9,21 +9,17 @@ fn t_cliargsparser_dash_capital_i_resolves_tilde_to_homedir() {
     env::set_var("HOME", tmp.path());
 
     let filename = "read.txt";
-    let arg = format!("~/{}", filename);
+    let arg = format!("~/{filename}");
 
     let check = |opts| {
         let args = CliArgsParser::new(opts);
         assert_eq!(args.readinfo_import_file, Some(tmp.path().join(filename)));
     };
 
-    check(vec![
-        "newsboat".to_string(),
-        "-I".to_string(),
-        arg.to_string(),
-    ]);
+    check(vec!["newsboat".into(), "-I".into(), arg.clone().into()]);
 
     check(vec![
-        "newsboat".to_string(),
-        "--import-from-file=".to_string() + &arg,
+        "newsboat".into(),
+        format!("--import-from-file={arg}").into(),
     ]);
 }
