@@ -9,21 +9,14 @@ fn t_cliargsparser_dash_d_resolves_tilde_to_homedir() {
     env::set_var("HOME", tmp.path());
 
     let filename = "newsboat.log";
-    let arg = format!("~/{}", filename);
+    let arg = format!("~/{filename}");
 
     let check = |opts| {
         let args = CliArgsParser::new(opts);
         assert_eq!(args.log_file, Some(tmp.path().join(filename)));
     };
 
-    check(vec![
-        "newsboat".to_string(),
-        "-d".to_string(),
-        arg.to_string(),
-    ]);
+    check(vec!["newsboat".into(), "-d".into(), arg.clone().into()]);
 
-    check(vec![
-        "newsboat".to_string(),
-        "--log-file=".to_string() + &arg,
-    ]);
+    check(vec!["newsboat".into(), format!("--log-file={arg}").into()]);
 }
