@@ -2,6 +2,7 @@
 
 #include <cassert>
 #include <cerrno>
+#include <cstdint>
 #include <cstdlib>
 #include <ctime>
 #include <curl/curl.h>
@@ -28,15 +29,15 @@
 #include "config.h"
 #include "configcontainer.h"
 #include "configexception.h"
-#include "configparser.h"
 #include "configpaths.h"
 #include "dbexception.h"
-#include "downloadthread.h"
 #include "exception.h"
 #include "feedhqapi.h"
 #include "feedhqurlreader.h"
+#include "formaction.h"
+#include "freshrssapi.h"
+#include "freshrssurlreader.h"
 #include "fileurlreader.h"
-#include "globals.h"
 #include "inoreaderapi.h"
 #include "inoreaderurlreader.h"
 #include "itemrenderer.h"
@@ -83,7 +84,7 @@ Controller::Controller(ConfigPaths& configpaths)
 	, refresh_on_start(false)
 	, api(0)
 	, configpaths(configpaths)
-	, queueManager(&cfg, &configpaths)
+	, queueManager(&cfg, configpaths.queue_file())
 {
 }
 
@@ -107,12 +108,19 @@ int Controller::run(const CliArgsParser& args)
 
 	refresh_on_start = args.refresh_on_start();
 
-	if (args.log_file().has_value()) {
-		Logger::set_logfile(args.log_file().value());
+	if (args.log_level().has_value()) {
+		logger::set_loglevel(args.log_level().value());
 	}
 
-	if (args.log_level().has_value()) {
-		Logger::set_loglevel(args.log_level().value());
+	if (args.log_file().has_value()) {
+		logger::set_logfile(args.log_file().value());
+	}
+
+	if (!args.log_file().has_value() && args.log_level().has_value()) {
+		const std::string date_time_string = utils::mt_strf_localtime("%Y-%m-%d_%H.%M.%S",
+				std::time(nullptr));
+		const std::string filename = "newsboat_" + date_time_string + ".log";
+		logger::set_logfile(filename);
 	}
 
 	if (!args.display_msg().empty()) {
@@ -136,8 +144,7 @@ int Controller::run(const CliArgsParser& args)
 
 	if (args.do_import()) {
 		LOG(Level::INFO, "Importing OPML file from %s", args.importfile());
-		import_opml(args.importfile(), configpaths.url_file());
-		return EXIT_SUCCESS;
+		return import_opml(args.importfile(), configpaths.url_file());
 	}
 
 	LOG(Level::INFO, "nl_langinfo(CODESET): %s", nl_langinfo(CODESET));
@@ -148,19 +155,6 @@ int Controller::run(const CliArgsParser& args)
 					PROGRAM_NAME,
 					utils::program_version())
 				<< std::endl;
-
-		fslock = std::unique_ptr<FsLock>(new FsLock());
-		pid_t pid;
-		if (!fslock->try_lock(configpaths.lock_file(), pid)) {
-			if (!args.cmds_to_execute().has_value()) {
-				std::cout << strprintf::fmt(
-						_("Error: an instance of %s is already running (PID: %u)"),
-						PROGRAM_NAME,
-						pid)
-					<< std::endl;
-			}
-			return EXIT_FAILURE;
-		}
 	}
 
 	if (!args.silent()) {
@@ -184,9 +178,10 @@ int Controller::run(const CliArgsParser& args)
 	cfgparser.register_handler("define-filter", filters);
 	cfgparser.register_handler("highlight", rxman);
 	cfgparser.register_handler("highlight-article", rxman);
+	cfgparser.register_handler("highlight-feed", rxman);
 
 	try {
-		cfgparser.parse_file("/etc/" PROGRAM_NAME "/config");
+		cfgparser.parse_file("/etc/" PACKAGE "/config");
 		cfgparser.parse_file(configpaths.config_file());
 	} catch (const ConfigException& ex) {
 		LOG(Level::ERROR,
@@ -207,17 +202,22 @@ int Controller::run(const CliArgsParser& args)
 	std::string cachefilepath = cfg.get_configvalue("cache-file");
 	if (cachefilepath.length() > 0 && !args.cache_file().has_value()) {
 		configpaths.set_cache_file(cachefilepath);
-		fslock = std::unique_ptr<FsLock>(new FsLock());
-		pid_t pid;
-		if (!fslock->try_lock(configpaths.lock_file(), pid)) {
+	}
+
+	pid_t pid;
+	std::string error;
+	if (!fslock.try_lock(configpaths.lock_file(), pid, error)) {
+		if (pid != 0) {
 			std::cout << strprintf::fmt(
 					_("Error: an instance of %s is "
-						"already running (PID: %u)"),
+						"already running (PID: %s)"),
 					PROGRAM_NAME,
-					pid)
+					std::to_string(pid))
 				<< std::endl;
-			return EXIT_FAILURE;
+		} else {
+			std::cout << _("Error: ") << error << std::endl;
 		}
+		return EXIT_FAILURE;
 	}
 
 	if (!args.silent()) {
@@ -249,19 +249,19 @@ int Controller::run(const CliArgsParser& args)
 	}
 
 	reloader =
-		std::unique_ptr<Reloader>(new Reloader(this, rsscache, &cfg));
+		std::unique_ptr<Reloader>(new Reloader(this, rsscache, cfg));
 
 	std::string type = cfg.get_configvalue("urls-source");
 	if (type == "local") {
 		urlcfg = new FileUrlReader(configpaths.url_file());
 	} else if (type == "opml") {
-		urlcfg = new OpmlUrlReader(&cfg);
+		urlcfg = new OpmlUrlReader(cfg);
 	} else if (type == "oldreader") {
-		api = new OldReaderApi(&cfg);
+		api = new OldReaderApi(cfg);
 		urlcfg = new OldReaderUrlReader(
 			&cfg, configpaths.url_file(), api);
 	} else if (type == "ttrss") {
-		api = new TtRssApi(&cfg);
+		api = new TtRssApi(cfg);
 		urlcfg = new TtRssUrlReader(configpaths.url_file(), api);
 	} else if (type == "newsblur") {
 		const auto cookies = cfg.get_configvalue("cookie-cache");
@@ -280,13 +280,37 @@ int Controller::run(const CliArgsParser& args)
 			return EXIT_FAILURE;
 		}
 
-		api = new NewsBlurApi(&cfg);
+		api = new NewsBlurApi(cfg);
 		urlcfg = new NewsBlurUrlReader(configpaths.url_file(), api);
 	} else if (type == "feedhq") {
-		api = new FeedHqApi(&cfg);
+		api = new FeedHqApi(cfg);
 		urlcfg = new FeedHqUrlReader(&cfg, configpaths.url_file(), api);
+	} else if (type == "freshrss") {
+		const auto freshrss_url = cfg.get_configvalue("freshrss-url");
+		if (freshrss_url.empty()) {
+			std::cerr <<
+				_("ERROR: You must set `freshrss-url` to use FreshRSS\n");
+			return EXIT_FAILURE;
+		}
+
+		const std::string user = cfg.get_configvalue("freshrss-login");
+		const std::string pass = cfg.get_configvalue("freshrss-password");
+		const std::string pass_file = cfg.get_configvalue("freshrss-passwordfile");
+		const std::string pass_eval = cfg.get_configvalue("freshrss-passwordeval");
+		const bool creds_set = !user.empty() &&
+			(!pass.empty() || !pass_file.empty() || !pass_eval.empty());
+		if (!creds_set) {
+			std::cerr <<
+				_("ERROR: You must set `freshrss-login` and one of `freshrss-password`, "
+					"`freshrss-passwordfile` or `freshrss-passwordeval` to use "
+					"FreshRSS\n");
+			return EXIT_FAILURE;
+		}
+
+		api = new FreshRssApi(cfg);
+		urlcfg = new FreshRssUrlReader(&cfg, configpaths.url_file(), api);
 	} else if (type == "ocnews") {
-		api = new OcNewsApi(&cfg);
+		api = new OcNewsApi(cfg);
 		urlcfg = new OcNewsUrlReader(configpaths.url_file(), api);
 	} else if (type == "miniflux") {
 		const auto miniflux_url = cfg.get_configvalue("miniflux-url");
@@ -300,17 +324,20 @@ int Controller::run(const CliArgsParser& args)
 		const std::string pass = cfg.get_configvalue("miniflux-password");
 		const std::string pass_file = cfg.get_configvalue("miniflux-passwordfile");
 		const std::string pass_eval = cfg.get_configvalue("miniflux-passwordeval");
-		const bool creds_set = !user.empty() &&
-			(!pass.empty() || !pass_file.empty() || !pass_eval.empty());
+		const std::string token = cfg.get_configvalue("miniflux-token");
+		const std::string token_file = cfg.get_configvalue("miniflux-tokenfile");
+		const std::string token_eval = cfg.get_configvalue("miniflux-tokeneval");
+		const bool creds_set = !token.empty()
+			|| !token_file.empty()
+			|| !token_eval.empty()
+			|| (!user.empty() && (!pass.empty() || !pass_file.empty() || !pass_eval.empty()));
 		if (!creds_set) {
 			std::cerr <<
-				_("ERROR: You must set `miniflux-login` and one of `miniflux-password`, "
-					"`miniflux-passwordfile` or `miniflux-passwordeval` to use "
-					"Miniflux\n");
+				_("ERROR: You must provide an API token or a login/password pair to use Miniflux. Please set the appropriate miniflux-* settings\n");
 			return EXIT_FAILURE;
 		}
 
-		api = new MinifluxApi(&cfg);
+		api = new MinifluxApi(cfg);
 		urlcfg = new MinifluxUrlReader(configpaths.url_file(), api);
 	} else if (type == "inoreader") {
 		const auto all_set = !cfg.get_configvalue("inoreader-app-id").empty()
@@ -321,7 +348,7 @@ int Controller::run(const CliArgsParser& args)
 			return EXIT_FAILURE;
 		}
 
-		api = new InoreaderApi(&cfg);
+		api = new InoreaderApi(cfg);
 		urlcfg = new InoreaderUrlReader(&cfg, configpaths.url_file(), api);
 	} else {
 		std::cerr << strprintf::fmt(_("ERROR: Unknown urls-source `%s'"),
@@ -340,8 +367,10 @@ int Controller::run(const CliArgsParser& args)
 			return EXIT_FAILURE;
 		}
 	}
-	urlcfg->reload();
-	if (!args.do_export() && !args.silent()) {
+	const auto error_message = urlcfg->reload();
+	if (error_message.has_value()) {
+		std::cout << error_message.value().message << std::endl << std::endl;
+	} else if (!args.do_export() && !args.silent()) {
 		std::cout << _("done.") << std::endl;
 	}
 
@@ -384,6 +413,11 @@ int Controller::run(const CliArgsParser& args)
 					_("It looks like you haven't configured any "
 						"feeds in your Miniflux account. Please do "
 						"so, and try again."));
+		} else if (type == "ocnews") {
+			msg = strprintf::fmt(
+					_("It looks like you haven't configured any feeds in your "
+						"Owncloud/Nextcloud account. Please do so, and try "
+						"again."));
 		} else {
 			assert(0); // shouldn't happen
 		}
@@ -391,15 +425,8 @@ int Controller::run(const CliArgsParser& args)
 		return EXIT_FAILURE;
 	}
 
-	if (!args.do_export() && !args.do_vacuum() && !args.silent()) {
-		std::cout << _("Loading articles from cache...");
-	}
 	if (args.do_vacuum()) {
 		std::cout << _("Opening cache...");
-	}
-	std::cout.flush();
-
-	if (args.do_vacuum()) {
 		std::cout << _("done.") << std::endl;
 		std::cout << _("Cleaning up cache thoroughly...");
 		std::cout.flush();
@@ -408,6 +435,11 @@ int Controller::run(const CliArgsParser& args)
 		return EXIT_SUCCESS;
 	}
 
+	if (!args.do_export() && !args.silent()) {
+		std::cout << _("Loading articles from cache...");
+	}
+	std::cout.flush();
+
 	unsigned int i = 0;
 	for (const auto& url : urlcfg->get_urls()) {
 		try {
@@ -443,6 +475,15 @@ int Controller::run(const CliArgsParser& args)
 		std::cout << _("done.") << std::endl;
 	}
 
+	if (args.do_cleanup()) {
+		std::cout << _("Cleaning up cache...");
+		std::cout.flush();
+		rsscache->cleanup_cache(feedcontainer.get_all_feeds(), true);
+		std::cout << _("done.") << std::endl;
+		return EXIT_SUCCESS;
+	}
+
+
 	// if configured, we fill all query feeds with some data; no need to
 	// sort it, it will be refilled when actually opening it.
 	if (cfg.get_configvalue_as_bool("prepopulate-query-feeds")) {
@@ -461,7 +502,7 @@ int Controller::run(const CliArgsParser& args)
 	feedcontainer.sort_feeds(cfg.get_feed_sort_strategy());
 
 	if (args.do_export()) {
-		export_opml();
+		export_opml(args.export_as_opml2());
 		return EXIT_SUCCESS;
 	}
 
@@ -493,8 +534,9 @@ int Controller::run(const CliArgsParser& args)
 	v->set_tags(tags);
 	v->set_cache(rsscache);
 
-	if (args.cmds_to_execute().has_value()) {
-		execute_commands(args.cmds_to_execute().value());
+	const auto cmds_to_execute = args.cmds_to_execute();
+	if (cmds_to_execute.size() >= 1) {
+		execute_commands(cmds_to_execute);
 		return EXIT_SUCCESS;
 	}
 
@@ -523,9 +565,21 @@ int Controller::run(const CliArgsParser& args)
 		std::cout.flush();
 	}
 	try {
-		rsscache->cleanup_cache(feedcontainer.get_all_feeds());
+		const auto unreachable_feeds = rsscache->cleanup_cache(
+				feedcontainer.get_all_feeds());
 		if (!args.silent()) {
 			std::cout << _("done.") << std::endl;
+			if (!unreachable_feeds.empty()) {
+				for (const auto& feed : unreachable_feeds) {
+					LOG(Level::USERERROR, "Unreachable feed found: %s", feed);
+				}
+
+				// Workaround for missing overload of strprintf::fmt for size_type on macOS.
+				std::uint64_t num_feeds = unreachable_feeds.size();
+				std::cout << strprintf::fmt(_("%" PRIu64 " unreachable feeds found. See "
+							"`cleanup-on-quit` in newsboat(1) for details."), num_feeds)
+					<< std::endl;
+			}
 		}
 	} catch (const DbException& e) {
 		LOG(Level::USERERROR, "Cleaning up cache failed: %s", e.what());
@@ -553,7 +607,7 @@ void Controller::mark_all_read(const std::string& feedurl)
 	try {
 		rsscache->mark_all_read(feedurl);
 	} catch (const DbException& e) {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error: couldn't mark all feeds read: %s"),
 				e.what()));
 		return;
@@ -596,6 +650,15 @@ void Controller::mark_all_read(unsigned int pos)
 	}
 
 	if (feed->is_query_feed()) {
+		if (api) {
+			std::vector<std::string> item_guids;
+			for (const auto& item : feed->items()) {
+				if (item->unread()) {
+					item_guids.push_back(item->guid());
+				}
+			}
+			api->mark_articles_read(item_guids);
+		}
 		rsscache->mark_all_read(feed);
 	} else {
 		rsscache->mark_all_read(feed->rssurl());
@@ -610,6 +673,14 @@ void Controller::mark_all_read(unsigned int pos)
 	feedcontainer.mark_all_feed_items_read(feed);
 }
 
+void Controller::mark_all_read(const std::vector<std::string>& item_guids)
+{
+	ScopeMeasure m("Controller::mark_all_read");
+	if (api) {
+		api->mark_articles_read(item_guids);
+	}
+}
+
 void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 	std::shared_ptr<RssFeed> newfeed,
 	unsigned int pos,
@@ -630,7 +701,26 @@ void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 	feed->set_tags(urlcfg->get_tags(oldfeed->rssurl()));
 	feed->set_order(oldfeed->get_order());
 	feedcontainer.replace_feed(pos, feed);
-	queueManager.autoenqueue(feed);
+
+	if (cfg.get_configvalue_as_bool("podcast-auto-enqueue")) {
+		const auto result = queueManager.autoenqueue(feed);
+		switch (result.status) {
+		case EnqueueStatus::QUEUED_SUCCESSFULLY:
+		case EnqueueStatus::URL_QUEUED_ALREADY:
+			// All is well, nothing to be done
+			break;
+		case EnqueueStatus::OUTPUT_FILENAME_USED_ALREADY:
+			v->get_statusline().show_error(
+				strprintf::fmt(_("Generated filename (%s) is used already."),
+					result.extra_info));
+			break;
+		case EnqueueStatus::QUEUE_FILE_OPEN_ERROR:
+			v->get_statusline().show_error(
+				strprintf::fmt(_("Failed to open queue file: %s."), result.extra_info));
+			break;
+		}
+	}
+
 	for (const auto& item : feed->items()) {
 		rsscache->update_rssitem_unread_and_enqueued(item, feed->rssurl());
 	}
@@ -641,27 +731,38 @@ void Controller::replace_feed(std::shared_ptr<RssFeed> oldfeed,
 	}
 }
 
-void Controller::import_opml(const std::string& opmlFile,
+int Controller::import_opml(const std::string& opmlFile,
 	const std::string& urlFile)
 {
 	auto urlReader = FileUrlReader(urlFile);
-	urlReader.reload(); // Load existing URLs
-
-	if (!opml::import(opmlFile, urlReader)) {
+	const auto error_message = urlReader.reload(); // Load existing URLs
+	if (error_message.has_value()) {
 		std::cout << strprintf::fmt(
-				_("An error occurred while parsing %s."), opmlFile)
+				_("Error importing OPML to urls file %s: %s"),
+				urlFile, error_message.value().message)
 			<< std::endl;
-		return;
+		return EXIT_FAILURE;
+	}
+
+	const auto import_error = opml::import(opmlFile, urlReader);
+	if (import_error.has_value()) {
+		std::cout << strprintf::fmt(
+				_("An error occurred while parsing %s: %s"),
+				opmlFile,
+				import_error.value())
+			<< std::endl;
+		return EXIT_FAILURE;
 	} else {
 		std::cout << strprintf::fmt(
 				_("Import of %s finished."), opmlFile)
 			<< std::endl;
+		return EXIT_SUCCESS;
 	}
 }
 
-void Controller::export_opml()
+void Controller::export_opml(bool version2)
 {
-	xmlDocPtr root = opml::generate(feedcontainer);
+	xmlDocPtr root = opml::generate(feedcontainer, version2);
 
 	xmlSaveCtxtPtr savectx = xmlSaveToFd(1, nullptr, 1);
 	xmlSaveDoc(savectx, root);
@@ -690,7 +791,7 @@ std::vector<std::shared_ptr<RssItem>> Controller::search_for_items(
 		}
 	} else {
 		items = rsscache->search_for_items(
-				query, (feed != nullptr ? feed->rssurl() : ""));
+				query, (feed != nullptr ? feed->rssurl() : ""), ign);
 		for (const auto& item : items) {
 			item->set_feedptr(
 				feedcontainer.get_feed_by_url(item->feedurl()));
@@ -699,15 +800,20 @@ std::vector<std::shared_ptr<RssItem>> Controller::search_for_items(
 	return items;
 }
 
-void Controller::enqueue_url(std::shared_ptr<RssItem> item,
+EnqueueResult Controller::enqueue_url(std::shared_ptr<RssItem> item,
 	std::shared_ptr<RssFeed> feed)
 {
-	queueManager.enqueue_url(item, feed);
+	return queueManager.enqueue_url(item, feed);
 }
 
 void Controller::reload_urls_file()
 {
-	urlcfg->reload();
+	const auto error_message = urlcfg->reload();
+	if (error_message.has_value()) {
+		v->get_statusline().show_message(error_message.value().message);
+		return;
+	}
+
 	std::vector<std::shared_ptr<RssFeed>> new_feeds;
 	unsigned int i = 0;
 
@@ -764,8 +870,8 @@ void Controller::edit_urls_file()
 
 	v->push_empty_formaction();
 	Stfl::reset();
-
 	utils::run_interactively(cmdline, "Controller::edit_urls_file");
+	v->drop_queued_input();
 
 	v->pop_current_formaction();
 
@@ -826,8 +932,23 @@ std::string Controller::write_temporary_item(std::shared_ptr<RssItem> item)
 void Controller::write_item(std::shared_ptr<RssItem> item,
 	const std::string& filename)
 {
-	std::fstream f;
-	f.open(filename.c_str(), std::fstream::out);
+	const std::string save_path = cfg.get_configvalue("save-path");
+	auto spath = save_path.back() == '/' ? save_path : save_path + "/";
+
+	std::string path;
+	switch (filename[0]) {
+	case '/':
+		path = filename;
+		break;
+	case '~':
+		path = utils::resolve_tilde(filename);
+		break;
+	default:
+		path = spath + filename;
+		break;
+	}
+
+	std::fstream f(path, std::fstream::out);
 	if (!f.is_open()) {
 		throw Exception(errno);
 	}
@@ -844,7 +965,7 @@ void Controller::import_read_information(const std::string& readinfofile)
 {
 	std::vector<std::string> guids;
 
-	std::ifstream f(readinfofile.c_str());
+	std::ifstream f(readinfofile);
 	std::string line;
 	getline(f, line);
 	if (!f.is_open()) {
@@ -861,8 +982,7 @@ void Controller::export_read_information(const std::string& readinfofile)
 {
 	std::vector<std::string> guids = rsscache->get_read_item_guids();
 
-	std::fstream f;
-	f.open(readinfofile.c_str(), std::fstream::out);
+	std::fstream f(readinfofile, std::fstream::out);
 	if (f.is_open()) {
 		for (const auto& guid : guids) {
 			f << guid << std::endl;
@@ -876,13 +996,13 @@ void Controller::update_config()
 
 	if (cfg.get_configvalue("error-log").length() > 0) {
 		try {
-			Logger::set_user_error_logfile(cfg.get_configvalue("error-log"));
+			logger::set_user_error_logfile(cfg.get_configvalue("error-log"));
 		} catch (const Exception& e) {
 			const std::string msg =
 				strprintf::fmt("Couldn't open %s: %s",
 					cfg.get_configvalue("error-log"),
 					e.what());
-			v->show_error(msg);
+			v->get_statusline().show_error(msg);
 			std::cerr << msg << std::endl;
 		}
 	}
@@ -893,13 +1013,13 @@ void Controller::load_configfile(const std::string& filename)
 	if (cfgparser.parse_file(filename)) {
 		update_config();
 	} else {
-		v->show_error(strprintf::fmt(
+		v->get_statusline().show_error(strprintf::fmt(
 				_("Error: couldn't open configuration file `%s'!"),
 				filename));
 	}
 }
 
-void Controller::dump_config(const std::string& filename)
+void Controller::dump_config(const std::string& filename) const
 {
 	std::vector<std::string> configlines;
 	cfg.dump_config(configlines);
@@ -910,8 +1030,7 @@ void Controller::dump_config(const std::string& filename)
 	filters.dump_config(configlines);
 	colorman.dump_config(configlines);
 	rxman.dump_config(configlines);
-	std::fstream f;
-	f.open(filename.c_str(), std::fstream::out);
+	std::fstream f(filename, std::fstream::out);
 	if (f.is_open()) {
 		for (const auto& line : configlines) {
 			f << line << std::endl;
