@@ -15,87 +15,99 @@ void FeedContainer::sort_feeds(const FeedSortStrategy& sort_strategy)
 
 	switch (sort_strategy.sm) {
 	case FeedSortMethod::NONE:
-		std::stable_sort(feeds.begin(),
-			feeds.end(),
-			[](std::shared_ptr<RssFeed> a,
-		std::shared_ptr<RssFeed> b) {
-			return a->get_order() < b->get_order();
+		std::stable_sort(
+			feeds.begin(), feeds.end(),
+		[&](std::shared_ptr<RssFeed> a, std::shared_ptr<RssFeed> b) {
+			if (sort_strategy.sd == SortDirection::ASC) {
+				return a->get_order() > b->get_order();
+			} else {
+				return a->get_order() < b->get_order();
+			}
 		});
 		break;
 	case FeedSortMethod::FIRST_TAG:
-		std::stable_sort(feeds.begin(),
-			feeds.end(),
-			[](std::shared_ptr<RssFeed> a,
-		std::shared_ptr<RssFeed> b) {
+		std::stable_sort(
+			feeds.begin(), feeds.end(),
+		[&](std::shared_ptr<RssFeed> a, std::shared_ptr<RssFeed> b) {
 			if (a->get_firsttag().length() == 0 ||
 				b->get_firsttag().length() == 0) {
-				return a->get_firsttag().length() >
-					b->get_firsttag().length();
+				bool result =
+					a->get_firsttag().length() > b->get_firsttag().length();
+				if (sort_strategy.sd == SortDirection::ASC) {
+					result = !result;
+				}
+				return result;
 			}
-			return utils::strnaturalcmp(a->get_firsttag().c_str(),
-					b->get_firsttag().c_str()) < 0;
+			const auto left = a->get_firsttag();
+			const auto right = b->get_firsttag();
+			bool result = utils::strnaturalcmp(left, right) < 0;
+			if (sort_strategy.sd == SortDirection::ASC) {
+				result = !result;
+			}
+			return result;
 		});
 		break;
 	case FeedSortMethod::TITLE:
-		std::stable_sort(feeds.begin(),
-			feeds.end(),
-			[](std::shared_ptr<RssFeed> a,
-		std::shared_ptr<RssFeed> b) {
-			return utils::strnaturalcmp(a->title().c_str(),
-					b->title().c_str()) < 0;
+		std::stable_sort(
+			feeds.begin(), feeds.end(),
+		[&](std::shared_ptr<RssFeed> a, std::shared_ptr<RssFeed> b) {
+			const auto left = a->title();
+			const auto right = b->title();
+			if (sort_strategy.sd == SortDirection::ASC) {
+				return utils::strnaturalcmp(left, right) > 0;
+			} else {
+				return utils::strnaturalcmp(left, right) < 0;
+			}
 		});
 		break;
 	case FeedSortMethod::ARTICLE_COUNT:
-		std::stable_sort(feeds.begin(),
-			feeds.end(),
-			[](std::shared_ptr<RssFeed> a,
-		std::shared_ptr<RssFeed> b) {
-			return a->total_item_count() <
-				b->total_item_count();
+		std::stable_sort(
+			feeds.begin(), feeds.end(),
+		[&](std::shared_ptr<RssFeed> a, std::shared_ptr<RssFeed> b) {
+			if (sort_strategy.sd == SortDirection::ASC) {
+				return a->total_item_count() > b->total_item_count();
+			} else {
+				return a->total_item_count() < b->total_item_count();
+			}
 		});
 		break;
 	case FeedSortMethod::UNREAD_ARTICLE_COUNT:
-		std::stable_sort(feeds.begin(),
-			feeds.end(),
-			[](std::shared_ptr<RssFeed> a,
-		std::shared_ptr<RssFeed> b) {
-			return a->unread_item_count() <
-				b->unread_item_count();
+		std::stable_sort(
+			feeds.begin(), feeds.end(),
+		[&](std::shared_ptr<RssFeed> a, std::shared_ptr<RssFeed> b) {
+			if (sort_strategy.sd == SortDirection::DESC) {
+				return a->unread_item_count() < b->unread_item_count();
+			} else {
+				return a->unread_item_count() > b->unread_item_count();
+			}
 		});
 		break;
 	case FeedSortMethod::LAST_UPDATED:
-		std::stable_sort(feeds.begin(),
-			feeds.end(),
-			[](std::shared_ptr<RssFeed> a,
-		std::shared_ptr<RssFeed> b) {
-			if (a->items().size() == 0 ||
-				b->items().size() == 0) {
-				return a->items().size() >
-					b->items().size();
+		std::stable_sort(
+			feeds.begin(), feeds.end(),
+		[&](std::shared_ptr<RssFeed> a, std::shared_ptr<RssFeed> b) {
+			if (a->items().size() == 0 || b->items().size() == 0) {
+				bool result = a->items().size() > b->items().size();
+				if (sort_strategy.sd == SortDirection::ASC) {
+					result = !result;
+				}
+				return result;
 			}
-			auto cmp =
-				[](std::shared_ptr<RssItem> a,
+			auto cmp = [](std::shared_ptr<RssItem> a,
 			std::shared_ptr<RssItem> b) {
 				return *a < *b;
 			};
-			auto& a_item =
-				*std::min_element(a->items().begin(),
-					a->items().end(),
-					cmp);
-			auto& b_item =
-				*std::min_element(b->items().begin(),
-					b->items().end(),
-					cmp);
-			return cmp(a_item, b_item);
-		});
-		break;
-	}
+			auto& a_item = *std::min_element(a->items().begin(),
+					a->items().end(), cmp);
+			auto& b_item = *std::min_element(b->items().begin(),
+					b->items().end(), cmp);
 
-	switch (sort_strategy.sd) {
-	case SortDirection::ASC:
-		std::reverse(feeds.begin(), feeds.end());
-		break;
-	case SortDirection::DESC:
+			if (sort_strategy.sd == SortDirection::DESC) {
+				return *a_item < *b_item;
+			} else {
+				return *b_item < *a_item;
+			}
+		});
 		break;
 	}
 }
