@@ -15,14 +15,18 @@ MinifluxUrlReader::MinifluxUrlReader(const std::string& url_file, RemoteApi* a)
 
 MinifluxUrlReader::~MinifluxUrlReader() {}
 
-void MinifluxUrlReader::reload()
+nonstd::optional<utils::ReadTextFileError> MinifluxUrlReader::reload()
 {
 	urls.clear();
 	tags.clear();
 	alltags.clear();
 
 	FileUrlReader ur(file);
-	ur.reload();
+	const auto error_message = ur.reload();
+	if (error_message.has_value()) {
+		LOG(Level::DEBUG, "Reloading failed: %s", error_message.value().message);
+		// Ignore errors for now: https://github.com/newsboat/newsboat/issues/1273
+	}
 
 	const std::vector<std::string>& file_urls(ur.get_urls());
 	for (const auto& url : file_urls) {
@@ -42,6 +46,8 @@ void MinifluxUrlReader::reload()
 			alltags.insert(tag);
 		}
 	}
+
+	return {};
 }
 
 std::string MinifluxUrlReader::get_source()
