@@ -23,6 +23,8 @@ extern "C" {
 #include <stfl.h>
 }
 
+#include "3rd-party/optional.hpp"
+
 #include "config.h"
 #include "colormanager.h"
 #include "controller.h"
@@ -142,7 +144,7 @@ bool View::run_commands(const std::vector<MacroCmd>& commands)
 		std::shared_ptr<FormAction> fa = get_current_formaction();
 		fa->prepare();
 		fa->draw_form();
-		if (!fa->process_op(command.op, true, &command.args)) {
+		if (!fa->process_op(command.op, command.args, BindingType::Macro)) {
 			// Operation failed, abort
 			return false;
 		}
@@ -236,7 +238,8 @@ int View::run()
 
 			// now we handle the operation to the
 			// formaction.
-			fa->process_op(op);
+			const std::vector<std::string> args;
+			fa->process_op(op, args);
 		}
 	}
 
@@ -295,10 +298,11 @@ std::string View::run_modal(std::shared_ptr<FormAction> f,
 			continue;
 		}
 
-		fa->process_op(op);
+		const std::vector<std::string> args;
+		fa->process_op(op, args);
 	}
 
-	if (value == "") {
+	if (value.empty()) {
 		return "";
 	} else {
 		return f->get_value(value);
@@ -566,7 +570,7 @@ void View::view_dialogs()
 	auto fa = get_current_formaction();
 	if (fa != nullptr && fa->id() != "dialogs") {
 		auto dialogs = std::make_shared<DialogsFormAction>(
-				this, dialogs_str, cfg);
+				this, dialogs_str, cfg, rxman);
 		dialogs->set_parent_formaction(fa);
 		apply_colors(dialogs);
 		dialogs->init();
@@ -600,7 +604,7 @@ void View::push_help()
 	current_formaction = formaction_stack_size() - 1;
 }
 
-void View::push_urlview(const std::vector<LinkPair>& links,
+void View::push_urlview(const Links& links,
 	std::shared_ptr<RssFeed>& feed)
 {
 	auto urlview = std::make_shared<UrlViewFormAction>(
@@ -613,23 +617,31 @@ void View::push_urlview(const std::vector<LinkPair>& links,
 	current_formaction = formaction_stack_size() - 1;
 }
 
-std::string View::run_filebrowser(const std::string& default_filename)
+nonstd::optional<std::string> View::run_filebrowser(const std::string& default_filename)
 {
 	auto filebrowser = std::make_shared<FileBrowserFormAction>(
 			this, filebrowser_str, cfg);
 	apply_colors(filebrowser);
 	filebrowser->set_default_filename(default_filename);
 	filebrowser->set_parent_formaction(get_current_formaction());
-	return run_modal(filebrowser, "filenametext");
+	std::string res = run_modal(filebrowser, "filenametext");
+	if (res.empty()) {
+		return nonstd::nullopt;
+	}
+	return res;
 }
 
-std::string View::run_dirbrowser()
+nonstd::optional<std::string> View::run_dirbrowser()
 {
 	auto dirbrowser = std::make_shared<DirBrowserFormAction>(
 			this, filebrowser_str, cfg);
 	apply_colors(dirbrowser);
 	dirbrowser->set_parent_formaction(get_current_formaction());
-	return run_modal(dirbrowser, "filenametext");
+	std::string res = run_modal(dirbrowser, "filenametext");
+	if (res.empty()) {
+		return nonstd::nullopt;
+	}
+	return res;
 }
 
 std::string View::select_tag(const std::string& current_tag)
