@@ -1,18 +1,20 @@
 #include "itemlistformaction.h"
 
 #include <fstream>
+#include <tuple>
 #include <unistd.h>
 
 #include "3rd-party/catch.hpp"
 #include "cache.h"
 #include "configpaths.h"
+#include "controller.h"
 #include "feedlistformaction.h"
 #include "itemlist.h"
 #include "keymap.h"
 #include "regexmanager.h"
 #include "rssfeed.h"
-#include "test-helpers/misc.h"
-#include "test-helpers/tempfile.h"
+#include "test_helpers/misc.h"
+#include "test_helpers/tempfile.h"
 
 using namespace newsboat;
 
@@ -22,7 +24,7 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile pagerfile;
+	test_helpers::TempFile pagerfile;
 
 	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
@@ -42,13 +44,13 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	Cache rsscache(":memory:", &cfg);
 	cfg.set_configvalue("pager", "cat %f > " + pagerfile.get_path());
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_link(test_url);
 	item->set_title(test_title);
 	item->set_author(test_author);
-	item->set_description(test_description);
+	item->set_description(test_description, "text/plain");
 	item->set_pubDate(test_pubDate);
 	item->set_unread(true);
 	feed->add_item(item);
@@ -59,9 +61,10 @@ TEST_CASE("OP_OPEN displays article using an external pager",
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_OPEN));
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist.process_op(OP_OPEN, args));
 
-	TestHelpers::assert_article_file_content(pagerfile.get_path(),
+	test_helpers::assert_article_file_content(pagerfile.get_path(),
 		test_title,
 		test_author,
 		test_pubDate_str,
@@ -79,7 +82,7 @@ TEST_CASE("OP_PURGE_DELETED purges previously deleted items",
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
 	RegexManager rxman;
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	feed->add_item(item);
 
@@ -90,13 +93,15 @@ TEST_CASE("OP_PURGE_DELETED purges previously deleted items",
 	itemlist.set_feed(feed);
 
 	SECTION("No items to purge") {
-		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED, args));
 		REQUIRE(feed->total_item_count() == 1);
 	}
 
 	SECTION("Deleted items are purged") {
 		item->set_deleted(true);
-		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PURGE_DELETED, args));
 		REQUIRE(feed->total_item_count() == 0);
 	}
 }
@@ -108,7 +113,7 @@ TEST_CASE(
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile browserfile;
+	test_helpers::TempFile browserfile;
 
 	const std::string test_url = "http://test_url";
 	std::string line;
@@ -120,7 +125,7 @@ TEST_CASE(
 	FilterContainer filters;
 	RegexManager rxman;
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_link(test_url);
 	item->set_unread(true);
@@ -131,7 +136,8 @@ TEST_CASE(
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
+	const std::vector<std::string> args;
+	itemlist.process_op(OP_OPENBROWSER_AND_MARK, args);
 	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
@@ -157,7 +163,7 @@ TEST_CASE(
 	FilterContainer filters;
 	RegexManager rxman;
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_link(test_url);
 	item->set_unread(true);
@@ -168,7 +174,8 @@ TEST_CASE(
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(OP_OPENBROWSER_AND_MARK);
+	const std::vector<std::string> args;
+	itemlist.process_op(OP_OPENBROWSER_AND_MARK, args);
 
 	REQUIRE(feed->unread_item_count() == 1);
 }
@@ -179,7 +186,7 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile browserfile;
+	test_helpers::TempFile browserfile;
 	const std::string test_url = "http://test_url";
 	std::string line;
 
@@ -190,7 +197,7 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 	FilterContainer filters;
 	RegexManager rxman;
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_link(test_url);
 	feed->add_item(item);
@@ -200,7 +207,43 @@ TEST_CASE("OP_OPENINBROWSER passes the url to the browser",
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
-	itemlist.process_op(OP_OPENINBROWSER);
+	const std::vector<std::string> args;
+	itemlist.process_op(OP_OPENINBROWSER, args);
+	std::ifstream browserFileStream(browserfile.get_path());
+
+	REQUIRE(std::getline(browserFileStream, line));
+	REQUIRE(line == test_url);
+}
+
+TEST_CASE("OP_OPENINBROWSER_NONINTERACTIVE passes the url to the browser",
+	"[ItemListFormAction]")
+{
+	ConfigPaths paths;
+	Controller c(paths);
+	newsboat::View v(&c);
+	test_helpers::TempFile browserfile;
+	const std::string test_url = "http://test_url";
+	std::string line;
+
+	ConfigContainer cfg;
+	cfg.set_configvalue("browser", "echo %u >> " + browserfile.get_path());
+
+	Cache rsscache(":memory:", &cfg);
+	FilterContainer filters;
+	RegexManager rxman;
+
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
+	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
+	item->set_link(test_url);
+	feed->add_item(item);
+
+	v.set_config_container(&cfg);
+	c.set_view(&v);
+
+	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
+	itemlist.set_feed(feed);
+	const std::vector<std::string> args;
+	itemlist.process_op(newsboat::OP_OPENINBROWSER_NONINTERACTIVE, args);
 	std::ifstream browserFileStream(browserfile.get_path());
 
 	REQUIRE(std::getline(browserFileStream, line));
@@ -213,7 +256,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile browserfile;
+	test_helpers::TempFile browserfile;
 	std::unordered_set<std::string> url_set;
 	const std::string test_url = "http://test_url";
 	std::string line;
@@ -226,7 +269,7 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 	FilterContainer filters;
 	RegexManager rxman;
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	for (int i = 0; i < itemCount; i++) {
 		std::shared_ptr<RssItem> item =
@@ -249,7 +292,8 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		openedItemsCount = 0;
@@ -272,7 +316,8 @@ TEST_CASE("OP_OPENALLUNREADINBROWSER passes the url list to the browser",
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
@@ -297,7 +342,7 @@ TEST_CASE(
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile browserfile;
+	test_helpers::TempFile browserfile;
 	std::unordered_set<std::string> url_set;
 	const std::string test_url = "http://test_url";
 	std::string line;
@@ -310,7 +355,7 @@ TEST_CASE(
 	FilterContainer filters;
 	RegexManager rxman;
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	for (unsigned int i = 0; i < itemCount; i++) {
 		std::shared_ptr<RssItem> item =
@@ -333,7 +378,8 @@ TEST_CASE(
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
@@ -357,7 +403,8 @@ TEST_CASE(
 		cfg.set_configvalue(
 			"max-browser-tabs", std::to_string(maxItemsToOpen));
 
-		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK);
+		const std::vector<std::string> args;
+		itemlist.process_op(OP_OPENALLUNREADINBROWSER_AND_MARK, args);
 
 		std::ifstream browserFileStream(browserfile.get_path());
 		if (browserFileStream.is_open()) {
@@ -384,7 +431,7 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
 	RegexManager rxman;
-	TestHelpers::TempFile urlFile;
+	test_helpers::TempFile urlFile;
 
 	const std::string test_url = "http://test_url";
 	std::string test_title = "Article Title";
@@ -400,13 +447,13 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 
 	item->set_link(test_url);
 	item->set_title(test_title);
 	item->set_author(test_author);
-	item->set_description(test_description);
+	item->set_description(test_description, "text/plain");
 	item->set_pubDate(test_pubDate);
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
@@ -416,9 +463,10 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 		cfg.set_configvalue(
 			"external-url-viewer", "tee > " + urlFile.get_path());
 
-		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS, args));
 
-		TestHelpers::assert_article_file_content(urlFile.get_path(),
+		test_helpers::assert_article_file_content(urlFile.get_path(),
 			test_title,
 			test_author,
 			test_pubDate_str,
@@ -429,11 +477,13 @@ TEST_CASE("OP_SHOWURLS shows the article's properties", "[ItemListFormAction]")
 	SECTION("internal url viewer") {
 		feed->add_item(item);
 		itemlist.set_feed(feed);
-		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS, args));
 	}
 
 	SECTION("no feed in formaction") {
-		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SHOWURLS, args));
 	}
 }
 
@@ -447,7 +497,7 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
 	RegexManager rxman;
-	TestHelpers::TempFile bookmarkFile;
+	test_helpers::TempFile bookmarkFile;
 	std::string line;
 	std::vector<std::string> bookmark_args;
 
@@ -460,7 +510,7 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	feed->set_title(feed_title);
 
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
@@ -476,14 +526,28 @@ TEST_CASE("OP_BOOKMARK pipes articles url and title to bookmark-command",
 		"bookmark-cmd", "echo > " + bookmarkFile.get_path());
 
 	bookmark_args.push_back(extra_arg);
-	REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, true, &bookmark_args));
 
-	std::ifstream browserFileStream(bookmarkFile.get_path());
+	auto checkOutput = [&] {
+		std::ifstream browserFileStream(bookmarkFile.get_path());
+
+		REQUIRE(std::getline(browserFileStream, line));
+		REQUIRE(line ==
+			test_url + separator + test_title + separator + extra_arg +
+			separator + feed_title);
+	};
+
+	SECTION("Macro") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, bookmark_args, BindingType::Macro));
+
+		checkOutput();
+	}
+
+	SECTION("Bind") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_BOOKMARK, bookmark_args, BindingType::Bind));
+
+		checkOutput();
+	}
 
-	REQUIRE(std::getline(browserFileStream, line));
-	REQUIRE(line ==
-		test_url + separator + test_title + separator + extra_arg +
-		separator + feed_title);
 }
 
 TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
@@ -502,7 +566,7 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
@@ -510,68 +574,37 @@ TEST_CASE("OP_EDITFLAGS arguments are added to an item's flags",
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	SECTION("Single flag") {
-		std::string flags = "G";
-		op_args.push_back(flags);
+	// <name, input, expected_result>
+	std::vector<std::tuple<std::string, std::string, std::string>> tests {
+		std::make_tuple("Single flag", "G", "G"),
+		std::make_tuple("Unordered flags", "abdefc", "abcdef"),
+		std::make_tuple("Duplicate flag in argument", "Abdddd", "Abd"),
+		std::make_tuple("Unauthorized values in arguments: Numbers", "Abd1236", "Abd"),
+		std::make_tuple("Unauthorized values in arguments: Symbolds", "Abd%^\\*;\'\"&~#{([-|`_/@)]=}$£€µ,;:!?./§", "Abd"),
+		std::make_tuple("Unauthorized values in arguments: Accents", "Abd¨^", "Abd"),
+		std::make_tuple("All possible flags at once", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz", "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"),
+	};
 
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == flags);
-	}
-
-	SECTION("Unordered flags") {
-		std::string flags = "abdefc";
-		std::string ordered_flags = "abcdef";
-		op_args.push_back(flags);
-
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == ordered_flags);
-	}
-
-	SECTION("Duplicate flag in argument") {
-		std::string flags = "Abd";
-		op_args.push_back(flags + "ddd");
-
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == flags);
-	}
-
-	SECTION("Unauthorized values in arguments") {
-		std::string flags = "Abd";
-		SECTION("Numbers") {
-			op_args.push_back(flags + "1236");
-
-			REQUIRE_NOTHROW(itemlist.process_op(
-					OP_EDITFLAGS, true, &op_args));
-			REQUIRE(item->flags() == flags);
-		}
-		SECTION("Symbols") {
-			op_args.push_back(flags +
-				"%^\\*;\'\"&~#{([-|`_/@)]=}$£€µ,;:!?./§");
-
-			REQUIRE_NOTHROW(itemlist.process_op(
-					OP_EDITFLAGS, true, &op_args));
-			REQUIRE(item->flags() == flags);
-		}
-		SECTION("Accents") {
-			op_args.push_back(flags + "¨^");
-
-			REQUIRE_NOTHROW(itemlist.process_op(
-					OP_EDITFLAGS, true, &op_args));
-			REQUIRE(item->flags() == flags);
+	SECTION("Macro") {
+		for (const auto& testCase : tests) {
+			DYNAMIC_SECTION("Test " << std::get<0>(testCase)) {
+				std::vector<std::string> args = op_args;
+				args.push_back(std::get<1>(testCase));
+				REQUIRE_NOTHROW( itemlist.process_op(OP_EDITFLAGS, args, BindingType::Macro));
+				REQUIRE(item->flags() == std::get<2>(testCase));
+			}
 		}
 	}
 
-	SECTION("All possible flags at once") {
-		std::string flags =
-			"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
-		op_args.push_back(flags);
-
-		REQUIRE_NOTHROW(
-			itemlist.process_op(OP_EDITFLAGS, true, &op_args));
-		REQUIRE(item->flags() == flags);
+	SECTION("Bind") {
+		for (const auto& testCase : tests) {
+			DYNAMIC_SECTION("Test " << std::get<0>(testCase)) {
+				std::vector<std::string> args = op_args;
+				args.push_back(std::get<1>(testCase));
+				REQUIRE_NOTHROW( itemlist.process_op(OP_EDITFLAGS, args, BindingType::Bind));
+				REQUIRE(item->flags() == std::get<2>(testCase));
+			}
+		}
 	}
 }
 
@@ -581,7 +614,7 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile saveFile;
+	test_helpers::TempFile saveFile;
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
@@ -604,28 +637,39 @@ TEST_CASE("OP_SAVE writes an article's attributes to the specified file",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_link(test_url);
 	item->set_title(test_title);
 	item->set_author(test_author);
 	item->set_pubDate(test_pubDate);
-	item->set_description(test_description);
+	item->set_description(test_description, "text/plain");
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, true, &op_args));
+	auto check = [&] {
+		test_helpers::assert_article_file_content(saveFile.get_path(),
+			test_title,
+			test_author,
+			test_pubDate_str,
+			test_url,
+			test_description);
+	};
+
+	SECTION("Macro") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, op_args, BindingType::Macro));
+		check();
+	}
+
+	SECTION("Bind") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_SAVE, op_args, BindingType::Bind));
+		check();
+	}
 
-	TestHelpers::assert_article_file_content(saveFile.get_path(),
-		test_title,
-		test_author,
-		test_pubDate_str,
-		test_url,
-		test_description);
 }
 
 TEST_CASE("OP_HELP command is processed", "[ItemListFormAction]")
@@ -642,14 +686,15 @@ TEST_CASE("OP_HELP command is processed", "[ItemListFormAction]")
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 
 	feed->add_item(item);
 
 	std::shared_ptr<ItemListFormAction> itemlist = v.push_itemlist(feed);
 
-	REQUIRE_NOTHROW(itemlist->process_op(OP_HELP));
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist->process_op(OP_HELP, args));
 }
 
 TEST_CASE("OP_HARDQUIT command is processed", "[ItemListFormAction]")
@@ -668,12 +713,13 @@ TEST_CASE("OP_HARDQUIT command is processed", "[ItemListFormAction]")
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_HARDQUIT));
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist.process_op(OP_HARDQUIT, args));
 }
 
 TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREV",
@@ -684,7 +730,7 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREV",
 	// updated.
 	ConfigPaths paths;
 	Controller c(paths);
-	TestHelpers::TempFile articleFile;
+	test_helpers::TempFile articleFile;
 	newsboat::View v(&c);
 	ConfigContainer cfg;
 	cfg.set_configvalue(
@@ -702,7 +748,7 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREV",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_title(first_article_title);
@@ -714,15 +760,16 @@ TEST_CASE("Navigate back and forth using OP_NEXT and OP_PREV",
 
 	std::shared_ptr<ItemListFormAction> itemlist = v.push_itemlist(feed);
 
-	REQUIRE_NOTHROW(itemlist->process_op(OP_NEXT));
-	itemlist->process_op(OP_SHOWURLS);
+	const std::vector<std::string> args;
+	REQUIRE_NOTHROW(itemlist->process_op(OP_NEXT, args));
+	itemlist->process_op(OP_SHOWURLS, args);
 
 	std::ifstream fileStream(articleFile.get_path());
 	std::getline(fileStream, line);
 	REQUIRE(line == prefix_title + second_article_title);
 
-	REQUIRE_NOTHROW(itemlist->process_op(OP_PREV));
-	itemlist->process_op(OP_SHOWURLS);
+	REQUIRE_NOTHROW(itemlist->process_op(OP_PREV, args));
+	itemlist->process_op(OP_SHOWURLS, args);
 
 	fileStream.seekg(0);
 	std::getline(fileStream, line);
@@ -744,7 +791,7 @@ TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	feed->add_item(item);
@@ -753,13 +800,15 @@ TEST_CASE("OP_TOGGLESHOWREAD switches the value of show-read-articles",
 
 	SECTION("True to False") {
 		v.get_cfg()->set_configvalue("show-read-articles", "yes");
-		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD, args));
 		REQUIRE_FALSE(v.get_cfg()->get_configvalue_as_bool(
 				"show-read-articles"));
 	}
 	SECTION("False to True") {
 		v.get_cfg()->set_configvalue("show-read-articles", "no");
-		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD));
+		const std::vector<std::string> args;
+		REQUIRE_NOTHROW(itemlist->process_op(OP_TOGGLESHOWREAD, args));
 		REQUIRE(v.get_cfg()->get_configvalue_as_bool(
 				"show-read-articles"));
 	}
@@ -771,7 +820,7 @@ TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 	ConfigPaths paths;
 	Controller c(paths);
 	newsboat::View v(&c);
-	TestHelpers::TempFile articleFile;
+	test_helpers::TempFile articleFile;
 	ConfigContainer cfg;
 	Cache rsscache(":memory:", &cfg);
 	FilterContainer filters;
@@ -794,26 +843,111 @@ TEST_CASE("OP_PIPE_TO pipes an article's content to an external command",
 	v.set_config_container(&cfg);
 	c.set_view(&v);
 
-	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache);
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
 
 	std::shared_ptr<RssItem> item = std::make_shared<RssItem>(&rsscache);
 	item->set_link(test_url);
 	item->set_title(test_title);
 	item->set_author(test_author);
 	item->set_pubDate(test_pubDate);
-	item->set_description(test_description);
+	item->set_description(test_description, "text/plain");
 
 	ItemListFormAction itemlist(&v, itemlist_str, &rsscache, filters, &cfg, rxman);
 
 	feed->add_item(item);
 	itemlist.set_feed(feed);
 
-	REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, true, &op_args));
+	auto check = [&] {
+		test_helpers::assert_article_file_content(articleFile.get_path(),
+			test_title,
+			test_author,
+			test_pubDate_str,
+			test_url,
+			test_description);
+	};
 
-	TestHelpers::assert_article_file_content(articleFile.get_path(),
-		test_title,
-		test_author,
-		test_pubDate_str,
-		test_url,
-		test_description);
+	SECTION("Macro") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, op_args, BindingType::Macro));
+		check();
+	}
+
+	SECTION("Bind") {
+		REQUIRE_NOTHROW(itemlist.process_op(OP_PIPE_TO, op_args, BindingType::Bind));
+		check();
+	}
+}
+
+TEST_CASE("OP_OPENINBROWSER does not result in itemlist invalidation",
+	"[ItemListFormAction]")
+{
+	ConfigPaths paths;
+	Controller c(paths);
+	ConfigContainer cfg;
+	KeyMap k(KM_NEWSBOAT);
+	newsboat::View v(&c);
+	v.set_config_container(&cfg);
+	v.set_keymap(&k);
+	Cache rsscache(":memory:", &cfg);
+	FilterContainer filters;
+	RegexManager rxman;
+
+	std::shared_ptr<RssItem> item1 = std::make_shared<RssItem>(&rsscache);
+	item1->set_link("https://example.com/1");
+	std::shared_ptr<RssItem> item2 = std::make_shared<RssItem>(&rsscache);
+	item2->set_link("https://example.com/2");
+	std::shared_ptr<RssItem> item3 = std::make_shared<RssItem>(&rsscache);
+	item3->set_link("https://example.com/3");
+
+	std::shared_ptr<RssFeed> feed = std::make_shared<RssFeed>(&rsscache, "");
+	feed->add_item(item1);
+	feed->add_item(item2);
+	feed->add_item(item3);
+
+	cfg.set_configvalue("mark-as-read-on-hover", "yes");
+	cfg.set_configvalue("show-read-articles", "no");
+	cfg.set_configvalue("browser", "echo");
+
+	SECTION("by default, all items are marked as unread") {
+		REQUIRE(item1->unread());
+		REQUIRE(item2->unread());
+		REQUIRE(item3->unread());
+	}
+
+	// The following sections have some interactions with STFL.
+	// We call `Stfl::reset()` to make sure the terminal is in a regular mode
+	// before calling Catch2 functions. Without the reset, Catch2 might output
+	// text while the terminal is in application mode, which makes it invisble
+	// when back in regular mode.
+	// The `View` object calls `Stfl::reset()` in its destructor so we can be
+	// sure we always return to the regular terminal mode, even when an
+	// exception is thrown.
+
+	SECTION("when entering ItemList, the first item is marked as 'read' due to 'mark-as-read-on-hover'") {
+		auto itemlist = v.push_itemlist(feed);
+		itemlist->prepare();
+
+		Stfl::reset();
+		REQUIRE_FALSE(item1->unread());
+		REQUIRE(item2->unread());
+		REQUIRE(item3->unread());
+	}
+
+	SECTION("executing 'open-in-browser' operation does not cause the next item to be marked as read") {
+		auto itemlist = v.push_itemlist(feed);
+		itemlist->prepare();
+
+		const std::vector<std::string> args;
+		bool success = itemlist->process_op(OP_OPENINBROWSER, args);
+		Stfl::reset();
+		REQUIRE(success);
+
+		itemlist->prepare();
+		Stfl::reset();
+
+		// Verify that following regression is fixed:
+		// https://github.com/newsboat/newsboat/issues/1292
+		REQUIRE_FALSE(item1->unread());
+		REQUIRE(item2->unread());
+		REQUIRE(item3->unread());
+	}
 }
